# -*- coding: utf-8 -*-

import os
import random


class Period:
    def __init__(self, name):
        self.name = name


class Company:
    def __init__(self, partner_id):
        self.partner_id = partner_id


class Country:
    def __init__(self, code):
        self.code = code


class Partner:
    def __init__(self, name, nif, country, aeat_registered=True):
        self.name = name
        self.vat = nif
        self.country = country
        self.aeat_registered = aeat_registered

    def sii_get_vat_type(self):
        return '02'


class Journal:
    def __init__(self, name, sii_description):
        self.name = name
        self.sii_description = sii_description


class FiscalPosition:
    def __init__(self, name):
        self.name = name


class Tax:
    def __init__(self, name, amount, type):
        self.name = name
        self.amount = amount
        self.type = type


class InvoiceTax:
    def __init__(self, name, base, tax_amount, tax_id):
        self.name = name
        self.base = base  # base imponible
        self.tax_amount = tax_amount
        self.tax_id = tax_id


class InvoiceLine:
    def __init__(self, price_subtotal, invoice_line_tax_id):
        self.price_subtotal = price_subtotal
        self.invoice_line_tax_id = invoice_line_tax_id


class Invoice:
    def __init__(self,
                 journal_id,
                 number,
                 invoice_type,
                 partner_id,
                 company_id,
                 amount_total,
                 amount_untaxed,
                 period_id,
                 date_invoice,
                 tax_line,
                 sii_registered,
                 rectificative_type,
                 fiscal_position,
                 invoice_line,
                 sii_in_clave_regimen_especial,
                 sii_out_clave_regimen_especial,
                 origin_date_invoice,
                 origin=None):
        self.journal_id = journal_id
        self.number = number
        self.type = invoice_type
        self.partner_id = partner_id
        self.company_id = company_id
        self.period_id = period_id
        self.amount_total = amount_total
        self.amount_untaxed = amount_untaxed
        self.origin_date_invoice = origin_date_invoice
        self.date_invoice = date_invoice
        self.tax_line = tax_line
        self.invoice_line = invoice_line
        self.fiscal_position = fiscal_position
        self.sii_registered = sii_registered
        self.origin = origin
        self.sii_in_clave_regimen_especial = sii_in_clave_regimen_especial
        self.sii_out_clave_regimen_especial = sii_out_clave_regimen_especial
        self.rectificative_type = rectificative_type
        # RECTIFICATIVE_TYPE_SELECTION = [  # TODO detectar tipus de rectificadora i fer una cosa u altra
        #     ('N', 'Normal'),
        #     ('R', 'Rectificative'),
        #     ('A', 'Nullifier'),
        #     ('B', 'Nullifier with substitution'),
        #     ('RA', 'Rectificative with differences')
        # ]


class DataGenerator:
    def __init__(self, invoice_registered=False, contraparte_registered=True):
        self.sii_registered = invoice_registered
        self.period = Period(name='12/2016')
        name_iva_21 = 'IVA 21%'
        name_iva_4 = 'IVA 4%'
        name_ibi = 'IBI 15%'
        name_iva_exento = 'IVA Exento'
        tax_ibi = Tax(name=name_ibi, amount=0.15, type='percent')
        tax_iva_21 = Tax(name=name_iva_21, amount=0.21, type='percent')
        tax_iva_4 = Tax(name=name_iva_21, amount=0.04, type='percent')
        tax_iva_exento = Tax(name=name_iva_exento, amount=0, type='percent')
        self.invoice_line = [
            InvoiceLine(price_subtotal=100, invoice_line_tax_id=[tax_iva_21]),
            InvoiceLine(
                price_subtotal=200, invoice_line_tax_id=[tax_iva_21, tax_ibi]
            ),
            InvoiceLine(price_subtotal=400, invoice_line_tax_id=[tax_iva_4]),
            InvoiceLine(price_subtotal=800, invoice_line_tax_id=[tax_ibi]),
            InvoiceLine(
                price_subtotal=1600, invoice_line_tax_id=[tax_iva_exento]
            )
        ]

        base_iva_21 = sum(
            [line.price_subtotal
             for line in self.invoice_line
             if tax_iva_21.amount
             in [tax.amount for tax in line.invoice_line_tax_id]]
        )
        base_iva_4 = sum(
            [line.price_subtotal
             for line in self.invoice_line
             if tax_iva_4.amount
             in [tax.amount for tax in line.invoice_line_tax_id]]
        )
        base_ibi = sum(
            [line.price_subtotal
             for line in self.invoice_line
             if tax_ibi.amount
             in [tax.amount for tax in line.invoice_line_tax_id]]
        )
        base_iva_exento = sum(
            [line.price_subtotal
             for line in self.invoice_line
             if tax_iva_exento.amount
             in [tax.amount for tax in line.invoice_line_tax_id]]
        )
        invoice_tax_iva_21 = InvoiceTax(
            name=name_iva_21, base=base_iva_21,
            tax_amount=base_iva_21 * tax_iva_21.amount, tax_id=tax_iva_21
        )
        invoice_tax_iva_4 = InvoiceTax(
            name=name_iva_4, base=base_iva_4,
            tax_amount=base_iva_4 * tax_iva_4.amount, tax_id=tax_iva_4
        )
        invoice_tax_ibi = InvoiceTax(
            name=name_ibi, base=base_ibi,
            tax_amount=base_ibi * tax_ibi.amount, tax_id=tax_ibi
        )
        invoice_tax_iva_exento = InvoiceTax(
            name=name_iva_exento, base=base_iva_exento,
            tax_amount=base_iva_exento * tax_iva_exento.amount,
            tax_id=tax_iva_exento
        )
        self.tax_line = [
            invoice_tax_iva_21, invoice_tax_iva_4, invoice_tax_ibi,
            invoice_tax_iva_exento
        ]
        spain = Country(code='ES')
        self.partner_invoice = Partner(
            name=os.environ.get('NOMBRE_CONTRAPARTE', u'Francisco García'),
            nif=os.environ.get('NIF_CONTRAPARTE', u'12345678T'),
            country=spain, aeat_registered=contraparte_registered
        )
        partner_company = Partner(
            name=os.environ.get('NOMBRE_TITULAR', u'Compañía Eléctrica S.A.'),
            nif=os.environ.get('NIF_TITULAR', '55555555T'), country=spain
        )
        self.company = Company(partner_id=partner_company)

        self.invoice_number = str(random.randrange(0, 100000)).zfill(5)
        self.origin_date_invoice = '2016-12-01'
        self.date_invoice = '2016-12-31'
        taxes_amount = sum([tax.tax_amount for tax in self.tax_line])
        base_amount = sum([line.price_subtotal for line in self.invoice_line])
        self.amount_untaxed = base_amount
        self.amount_total = taxes_amount + base_amount
        self.fiscal_position = FiscalPosition(
            name=u'Régimen Islas Canarias'
        )
        self.sii_in_clave_regimen_especial = '01'
        self.sii_out_clave_regimen_especial = '01'

    def get_in_invoice(self):
        journal = Journal(
            name='Factura de Energía Recibida',
            sii_description='Descripción Facturas Recibidas'
        )

        invoice = Invoice(
            invoice_type='in_invoice',
            journal_id=journal,
            rectificative_type='N',
            number='FRecib{}'.format(self.invoice_number),
            origin='FRecibOrigen{}'.format(self.invoice_number),
            partner_id=self.partner_invoice,
            company_id=self.company,
            amount_total=self.amount_total,
            amount_untaxed=self.amount_untaxed,
            period_id=self.period,
            origin_date_invoice=self.origin_date_invoice,
            date_invoice=self.date_invoice,
            tax_line=self.tax_line,
            invoice_line=self.invoice_line,
            sii_registered=self.sii_registered,
            fiscal_position=self.fiscal_position,
            sii_in_clave_regimen_especial=self.sii_in_clave_regimen_especial,
            sii_out_clave_regimen_especial=self.sii_out_clave_regimen_especial
        )
        return invoice

    def get_out_invoice(self):
        journal = Journal(
            name='Factura de Energía Emitida',
            sii_description='Descripción Facturas Emitidas'
        )

        invoice = Invoice(
            invoice_type='out_invoice',
            journal_id=journal,
            rectificative_type='N',
            number='FEmit{}'.format(self.invoice_number),
            partner_id=self.partner_invoice,
            company_id=self.company,
            amount_total=self.amount_total,
            amount_untaxed=self.amount_untaxed,
            period_id=self.period,
            origin_date_invoice=self.origin_date_invoice,
            date_invoice=self.date_invoice,
            tax_line=self.tax_line,
            invoice_line=self.invoice_line,
            sii_registered=self.sii_registered,
            fiscal_position=self.fiscal_position,
            sii_in_clave_regimen_especial=self.sii_in_clave_regimen_especial,
            sii_out_clave_regimen_especial=self.sii_out_clave_regimen_especial
        )
        return invoice

    def get_in_refund_invoice(self):
        journal = Journal(
            name='Factura de Energía Rectificativa Recibida',
            sii_description='Descripción Facturas Rectificativas Recibidas'
        )

        invoice = Invoice(
            invoice_type='in_refund',
            journal_id=journal,
            rectificative_type='R',
            number='FRectRecib{}'.format(self.invoice_number),
            origin='FRectRecibOrigen{}'.format(self.invoice_number),
            partner_id=self.partner_invoice,
            company_id=self.company,
            amount_total=self.amount_total,
            amount_untaxed=self.amount_untaxed,
            period_id=self.period,
            origin_date_invoice=self.origin_date_invoice,
            date_invoice=self.date_invoice,
            tax_line=self.tax_line,
            invoice_line=self.invoice_line,
            sii_registered=self.sii_registered,
            fiscal_position=self.fiscal_position,
            sii_in_clave_regimen_especial=self.sii_in_clave_regimen_especial,
            sii_out_clave_regimen_especial=self.sii_out_clave_regimen_especial
        )
        return invoice

    def get_out_refund_invoice(self):
        journal = Journal(
            name='Factura de Energía Rectificativa Emitida',
            sii_description='Descripción Facturas Rectificativas Emitidas'
        )

        invoice = Invoice(
            invoice_type='out_refund',
            journal_id=journal,
            rectificative_type='R',
            number='FRectEmit{}'.format(self.invoice_number),
            partner_id=self.partner_invoice,
            company_id=self.company,
            amount_total=self.amount_total,
            amount_untaxed=self.amount_untaxed,
            period_id=self.period,
            origin_date_invoice=self.origin_date_invoice,
            date_invoice=self.date_invoice,
            tax_line=self.tax_line,
            invoice_line=self.invoice_line,
            sii_registered=self.sii_registered,
            fiscal_position=self.fiscal_position,
            sii_in_clave_regimen_especial=self.sii_in_clave_regimen_especial,
            sii_out_clave_regimen_especial=self.sii_out_clave_regimen_especial
        )
        return invoice



# -*- coding: UTF-8 -*-
import logging

logging.basicConfig(level=logging.DEBUG)

from sii.server import *
from spec.testing_data import DataGenerator
from pprintpp import pprint
# Enviament de factures
data_gen = DataGenerator(contraparte_registered=True, invoice_registered=False)
in_invoice = data_gen.get_in_invoice()
out_invoice = data_gen.get_out_invoice()
in_refund = data_gen.get_in_refund_invoice()
out_refund = data_gen.get_out_refund_invoice()
s = SiiService('/home/gdalmau/Documents/SII/SSL/client.crt',
               '/home/gdalmau/Documents/SII/SSL/client.key',
               'https://sii-proxy.gisce.net:4443')
invoice = out_invoice
res = s.send(invoice)
pprint(res)

# # Consulta de factures
# s = SiiService('/home/gdalmau/Documents/SII/SSL/client.crt',
#                '/home/gdalmau/Documents/SII/SSL/client.key',
#                'https://sii-proxy.gisce.net:4443')
# res = s.query_invoice('out_invoice', '40351345F', 'Eduard Carreras Nadal',
#                       '2016', '12')
# from pprintpp import pprint
# pprint(res)

# Validació de NIF
# s = IDService('/home/miquel/Documents/SII/client_ssl/client.crt',
#               '/home/miquel/Documents/SII/client_ssl/client.key',
#               'https://sii-proxy.gisce.net:4443/nifs')
# d2 = [{'vat': '46724290C',
#        'name': 'Guillem Julia Blasi'},
#       {'vat': '40351345F',
#        'name': 'Eduard Carreras Nadal'},
#       {'vat': '82641631D',
#        'name': 'Dalmau Vila Dalmau'}
#       ]
# d1 = [{'vat': '46724290C',
#       'name': 'Guillem Julia Blasi'}]
# res = s.invalid_ids(d2, 1)
# print res



from expects import *
with description('1'):
    with before.all:
        self.a = 'a'

    with it('2'):
        expect(self.a).to(equal('a'))

    with context('3'):
        with before.all:
            self.a = 'b'

        with it('4'):
            expect(self.a).to(equal('b'))
    with it('5'):
        expect(self.a).to(equal('a'))


# coding=utf-8
import re
from unidecode import unidecode

from sii import __SII_VERSION__
from sii.models import invoices_record

SIGN = {'B': -1, 'A': -1, 'N': 1, 'R': 1}


def get_iva_values(invoice, in_invoice, is_export=False, is_import=False):
    vals = {
        'sujeta_a_iva': False,
        'detalle_iva': [],
        'no_sujeta_a_iva': False,
        'iva_exento': False,
        'iva_no_exento': False,
        'detalle_iva_exento': {'BaseImponible': 0},
        'importe_no_sujeto': 0
    }

    invoice_total = invoice.amount_total

    for inv_tax in invoice.tax_line:
        if 'iva' in inv_tax.name.lower():
            vals['sujeta_a_iva'] = True

            invoice_total -= (abs(inv_tax.tax_amount) + abs(inv_tax.base))

            is_iva_exento = (
                inv_tax.tax_id.amount == 0 and inv_tax.tax_id.type == 'percent'
            )
            if not is_export and not is_import and is_iva_exento:
                vals['iva_exento'] = True
                vals['detalle_iva_exento']['BaseImponible'] += inv_tax.base
            else:
                sign = SIGN[invoice.rectificative_type]
                iva = {
                    'BaseImponible': sign * abs(inv_tax.base),
                    'TipoImpositivo': inv_tax.tax_id.amount * 100
                }
                if in_invoice:
                    iva['CuotaSoportada'] = sign * abs(inv_tax.tax_amount)
                else:
                    iva['CuotaRepercutida'] = sign * abs(inv_tax.tax_amount)
                vals['iva_no_exento'] = True
                vals['detalle_iva'].append(iva)

    invoice_total = round(invoice_total, 2)
    if invoice_total > 0:
        vals['no_sujeta_a_iva'] = True
        vals['importe_no_sujeto'] = invoice_total

    return vals


def get_contraparte(partner, in_invoice):
    vat_type = partner.sii_get_vat_type()
    contraparte = {'NombreRazon': partner.name}

    if not partner.aeat_registered and not in_invoice:
        contraparte['IDOtro'] = {
            'CodigoPais': partner.country.code,
            'IDType': '07',
            'ID': partner.vat
        }
    else:
        if vat_type == '02':
            contraparte['NIF'] = partner.vat
        else:
            contraparte['IDOtro'] = {
                'CodigoPais': partner.country.code,
                'IDType': vat_type,
                'ID': partner.vat
            }

    return contraparte


def get_factura_emitida_tipo_desglose(invoice):

    if invoice.sii_out_clave_regimen_especial == '02':  # Exportación
        iva_values = get_iva_values(invoice, in_invoice=False, is_export=True)

        if iva_values['sujeta_a_iva']:
            entrega = {
                'Sujeta': {
                    'NoExenta': {
                        'TipoNoExenta': 'S1',
                        'DesgloseIVA': {
                            'DetalleIVA': iva_values['detalle_iva']
                        }
                    }
                }
            }
        else:
            entrega = {
                'Sujeta': {
                    'Exenta': iva_values['detalle_iva_exento']
                }
            }
            # Exenta por el artículo 21
            entrega['Sujeta']['Exenta']['CausaExencion'] = 'E2'

        tipo_desglose = {
            'DesgloseTipoOperacion': {
                'Entrega': entrega
            }
        }
    else:
        iva_values = get_iva_values(invoice, in_invoice=False)
        desglose = {}

        if iva_values['sujeta_a_iva']:
            desglose['Sujeta'] = {}
            if iva_values['iva_exento']:
                desglose['Sujeta']['Exenta'] = iva_values['detalle_iva_exento']
            if iva_values['iva_no_exento']:
                desglose['Sujeta']['NoExenta'] = {
                    'TipoNoExenta': 'S1',
                    'DesgloseIVA': {
                        'DetalleIVA': iva_values['detalle_iva']
                    }
                }
        if iva_values['no_sujeta_a_iva']:
            importe_no_sujeto = iva_values['importe_no_sujeto']

            fp = invoice.fiscal_position
            if fp and 'islas canarias' in unidecode(fp.name.lower()):
                desglose['NoSujeta'] = {
                    'ImporteTAIReglasLocalizacion': importe_no_sujeto
                }
            else:
                desglose['NoSujeta'] = {
                    'ImportePorArticulos7_14_Otros': importe_no_sujeto
                }

        partner_vat = invoice.partner_id.vat
        partner_vat_starts_with_n = partner_vat and partner_vat.upper().startswith('N')
        has_id_otro = invoice.partner_id.sii_get_vat_type() != '02'
        if has_id_otro or partner_vat_starts_with_n:
            tipo_desglose = {
                'DesgloseTipoOperacion': {
                    'PrestacionServicios': desglose
                }
            }
        else:
            tipo_desglose = {
                'DesgloseFactura': desglose
            }

    return tipo_desglose


def get_factura_emitida(invoice):

    factura_expedida = {
        'TipoFactura': 'R4' if invoice.rectificative_type == 'R' else 'F1',
        'ClaveRegimenEspecialOTrascendencia':
            invoice.sii_out_clave_regimen_especial,
        'ImporteTotal': SIGN[invoice.rectificative_type] * invoice.amount_total,
        'DescripcionOperacion':
            invoice.journal_id.sii_description or invoice.journal_id.name,
        'Contraparte': get_contraparte(invoice.partner_id, in_invoice=False),
        'TipoDesglose': get_factura_emitida_tipo_desglose(invoice)
    }

    return factura_expedida


def get_factura_recibida(invoice):
    cuota_deducible = 0

    # Factura correspondiente a una importación (informada sin asociar a un DUA
    if invoice.sii_in_clave_regimen_especial == '13':
        iva_values = get_iva_values(invoice, in_invoice=True, is_import=True)
    else:
        iva_values = get_iva_values(invoice, in_invoice=True)

    if iva_values['sujeta_a_iva'] and iva_values['iva_no_exento']:
        desglose_factura = {  # TODO to change
            # 'InversionSujetoPasivo': {
            #     'DetalleIVA': iva_values['detalle_iva']
            # },
            'DesgloseIVA': {
                'DetalleIVA': iva_values['detalle_iva']
            }
        }

        for detalle_iva in iva_values['detalle_iva']:
            cuota_deducible += detalle_iva['CuotaSoportada']
    else:
        desglose_factura = {
            'DesgloseIVA': {
                'DetalleIVA': [{
                    'BaseImponible': invoice.amount_untaxed
                }]
            }
        }

    factura_recibida = {
        'TipoFactura': 'R4' if invoice.rectificative_type == 'R' else 'F1',
        'ClaveRegimenEspecialOTrascendencia':
            invoice.sii_in_clave_regimen_especial,
        'ImporteTotal': SIGN[invoice.rectificative_type] * invoice.amount_total,
        'DescripcionOperacion':
            invoice.journal_id.sii_description or invoice.journal_id.name,
        'Contraparte': get_contraparte(invoice.partner_id, in_invoice=True),
        'DesgloseFactura': desglose_factura,
        'CuotaDeducible': cuota_deducible,
        'FechaRegContable': invoice.date_invoice
    }

    return factura_recibida


def get_header(invoice):
    cabecera = {
        'IDVersionSii': __SII_VERSION__,
        'Titular': {
            'NombreRazon': invoice.company_id.partner_id.name,
            'NIF': invoice.company_id.partner_id.vat
        },
        'TipoComunicacion': 'A0' if not invoice.sii_registered else 'A1'
    }

    return cabecera


def get_factura_rectificativa_fields():
    rectificativa_fields = {
        'TipoRectificativa': 'S',  # Por sustitución
        'ImporteRectificacion': {
            'BaseRectificada': 0,
            'CuotaRectificada': 0
        }
    }

    return rectificativa_fields


def get_factura_emitida_dict(invoice, rectificativa=False):
    obj = {
        'SuministroLRFacturasEmitidas': {
            'Cabecera': get_header(invoice),
            'RegistroLRFacturasEmitidas': {
                'PeriodoImpositivo': {
                    'Ejercicio': invoice.period_id.name[3:7],
                    'Periodo': invoice.period_id.name[0:2]
                },
                'IDFactura': {
                    'IDEmisorFactura': {
                        'NIF': invoice.company_id.partner_id.vat
                    },
                    'NumSerieFacturaEmisor': invoice.number,
                    'FechaExpedicionFacturaEmisor': invoice.date_invoice
                },
                'FacturaExpedida': get_factura_emitida(invoice)
            }
        }
    }

    if rectificativa:
        vals = get_factura_rectificativa_fields()

        (
            obj['SuministroLRFacturasEmitidas']['RegistroLRFacturasEmitidas']
            ['FacturaExpedida']
        ).update(vals)

    return obj


def get_factura_recibida_dict(invoice, rectificativa=False):
    obj = {
        'SuministroLRFacturasRecibidas': {
            'Cabecera': get_header(invoice),
            'RegistroLRFacturasRecibidas': {
                'PeriodoImpositivo': {
                    'Ejercicio': invoice.period_id.name[3:7],
                    'Periodo': invoice.period_id.name[0:2]
                },
                'IDFactura': {
                    'IDEmisorFactura': {
                        'NIF': invoice.partner_id.vat
                    },
                    'NumSerieFacturaEmisor': invoice.origin,
                    'FechaExpedicionFacturaEmisor': invoice.origin_date_invoice
                },
                'FacturaRecibida': get_factura_recibida(invoice)
            }
        }
    }

    if rectificativa:
        vals = get_factura_rectificativa_fields()

        (
            obj['SuministroLRFacturasRecibidas']['RegistroLRFacturasRecibidas']
            ['FacturaRecibida']
        ).update(vals)

    return obj


def refactor_nifs(invoice):
    for partner in (invoice.partner_id, invoice.company_id.partner_id):
        partner.vat = re.sub('^ES', '', partner.vat.upper())


def format_errors(errors_list):
    formatted_errors = '\n'.join(['-> {}'.format(err) for err in errors_list])
    return formatted_errors


class SII(object):
    def __init__(self, invoice):
        self.invoice = invoice
        refactor_nifs(self.invoice)
        rectificativa = invoice.rectificative_type == 'R'
        if invoice.type.startswith('in'):
            self.invoice_model = invoices_record.SuministroFacturasRecibidas()
            self.invoice_dict = get_factura_recibida_dict(
                self.invoice, rectificativa=rectificativa
            )
        elif invoice.type.startswith('out'):
            self.invoice_model = invoices_record.SuministroFacturasEmitidas()
            self.invoice_dict = get_factura_emitida_dict(
                self.invoice, rectificativa=rectificativa
            )
        else:
            raise AttributeError(
                'Valor desconocido en el tipo de factura: {}'.format(
                    invoice.type
                )
            )

    def get_validation_errors_list(self, errors):
        error_messages = []

        for key, values in errors.items():
            if isinstance(values, dict):
                error_messages += self.get_validation_errors_list(values)
            else:
                error_messages += ['{}: {}'.format(key, val) for val in values]

        return error_messages

    def validate_invoice(self):

        res = {}

        errors = self.invoice_model.validate(self.invoice_dict)

        res['successful'] = False if errors else True
        if errors:
            errors_list = self.get_validation_errors_list(errors)
            res['errors'] = errors_list

        return res

    def generate_object(self, print_obj=False):

        validation_values = self.validate_invoice()
        if not validation_values['successful']:
            raise Exception(
                'Errors were found while trying to validate the data:',
                validation_values['errors']
            )
        if print_obj:
            print '\n'
            print '===== OBJETO PARA GENERAR DUMP ' + self.invoice.type + ' ============='
            from pprintpp import pprint, pformat
            pprint(self.invoice_dict)
            print '=================================================\n'

        res = self.invoice_model.dump(self.invoice_dict)
        if res.errors:
            raise Exception(
                'Errors were found while trying to generate the dump:',
                res.errors
            )

        return res.data



# coding=utf-8
from marshmallow import fields, Schema, validate
from sii import invoices_record_models

class SingleModel(Schema):
    str = fields.String(validate=validate.Length(max=5, error='STRING MASSSA LLARG COLLONS'))

new_model = SingleModel()
new_obj = {'str': '123456'}
errors = new_model.validate(new_obj)
print '\n\n\n************* Errors SINGLE MODEL 1: ', errors
print '\n'
print '========================='
print 'EL SINGLE MODEL 1 de ejemplo es:\n'
from pprintpp import pprint
single_model_dump = new_model.dump(new_obj)
pprint(new_model.load(new_obj).data)
# pprint(single_model_dump.data)
print '========================='


new_obj2 = {'str': '123'}
errors = new_model.validate(new_obj2)
print '\n\n\n************* Errors SINGLE MODEL 2: ', errors
print '\n'
print '========================='
print 'EL SINGLE MODEL 2 de ejemplo es:\n'
from pprintpp import pprint
single_model_dump = new_model.dump(new_obj2)
pprint(new_model.load(new_obj2).data)
pprint(single_model_dump.data)
print '========================='

titular = {
    'NombreRazon': 'wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww',
}
titular_model = invoices_record_models.Titular()
errors = titular_model.validate(titular)
print '\n\n\n************* Errors TITULAR MODEL: ', errors
print '\n'
print '========================='
print 'EL TITULAR de ejemplo es:\n'
titular_dump = titular_model.dump(titular)
pprint(titular_dump.data)
print '========================='


class BlobSchema(Schema):
    chunks = fields.List(fields.String)

schema = BlobSchema()
obj = {
    'chunks': [
        'hola', 'adeu'
    ]
}
print '*********************'
print pprint(schema.dump(obj).data)



from sii.resource import SII
from sii.models.invoices_record import CRE_FACTURAS_EMITIDAS
from expects import *
from spec.testing_data import DataGenerator

# data_gen = DataGenerator()
# invoice = data_gen.get_out_invoice()
# invoice_obj = SII.generate_object(invoice)
# cabecera = (
#     invoice_obj['SuministroLRFacturasEmitidas']['Cabecera']
# )

import os
from expects import *

os.environ['NIF_TITULAR'] = 'ES11111111T'
os.environ['NIF_CONTRAPARTE'] = 'esES654321P'
new_data_gen = DataGenerator()
nifs_test_invoice = new_data_gen.get_out_invoice()
nif_invoice = nifs_test_invoice.company_id.partner_id.vat


nifs_test_obj = SII.generate_object(nifs_test_invoice)
nif_generated = nifs_test_obj['SuministroLRFacturasEmitidas']['Cabecera']['Titular']['NIF']

expect(nif_generated).to(equal(nif_invoice))

expect(
    nifs_test_obj['SuministroLRFacturasEmitidas']
    ['RegistroLRFacturasEmitidas']['FacturaExpedida']
    ['Contraparte']['NIF']
).to(
    equal(nifs_test_invoice.company_id.partner_id.vat[2:])
)



from marshmallow import Schema, fields, validate

class WorkingModel(Schema):
    str = fields.String(validate=validate.Length(max=5, error='String is too long'))

class NonWorkingModel(Schema):
    str = fields.String(
        load_from='sample_string',
        dump_to='SampleString',
        validate=validate.Length(max=5, error='String is too long')
    )

working_model = WorkingModel()
working_obj = {'str': '123456'}
errors = working_model.validate(working_obj)
print errors
# Output:
# {'str': ['String is too long']}


non_working_model = NonWorkingModel()
non_working_obj = {'sample_string': '123456'}
errors = non_working_model.validate(non_working_obj)
print errors
obj = non_working_model.dump(non_working_obj)
print obj.data
print obj.errors
print "===="
obj2 = non_working_model.load(non_working_obj)
print obj2.errors
print obj2.data
# Output:
# {}

from invoices_record_models import IdentificacionFactura
from datetime import datetime
m = IdentificacionFactura()
obj = {
        'FechaExpedicionFacturaEmisor': datetime.strptime(
            '2016-03-25', '%Y-%m-%d'
        )
    }
dumbo = m.dump(
    obj
)
errors = m.validate(obj)
print dumbo.data



from marshmallow import Schema, fields, validate


class NestedClass(Schema):
    DetalleIVA = fields.Integer()


class NestedListTest(Schema):
    DesgloseIVA = fields.List(fields.Nested(NestedClass), required=True)


class ListTest(Schema):
    Llista = fields.List(fields.Integer, required=True)


list_model = ListTest()
obj = {
    'Llista': [1, 2, 3]
}
errors = list_model.validate(obj)
if errors:
    raise Exception('Errors found 1', errors)
else:
    print 'Data 1'
    dumpo = list_model.dump(obj).data
    print dumpo
    from dicttoxml import dicttoxml

    xml_from_dict = dicttoxml(dumpo, root=False, attr_type=False)
    print xml_from_dict
print ' """""""""""""""""""""""""""""'
nested_list_model = NestedListTest()
new_obj = {
    'DesgloseIVA': [
        {
            'DetalleIVA': 1
        },
        {
            'DetalleIVA': 2
        }
    ]
}
errors = nested_list_model.validate(new_obj)
if errors:
    raise Exception('Errors found 2', errors)
else:
    print 'Data 2'
    from pprintpp import pprint

    donald_dump = nested_list_model.dump(new_obj).data
    print donald_dump
    from lxml import etree
    from lxml import objectify

    xml_from_dict = dicttoxml(donald_dump, root=False, attr_type=False)
    print xml_from_dict
    xml_from_dict = xml_from_dict.replace('<item>', '')
    xml_from_dict = xml_from_dict.replace('</item>', '')
    print xml_from_dict
print ' *9***************** PROVA 3 ************'
nested_list_model = NestedListTest()
new_obj = {
    'DesgloseIVA': {
        'DetalleIVA': [
            {
                'Base': 1
            },
            {
                'Base': 2
            }
        ]
    }
}


class BaseImponible(Schema):
    Base = fields.Integer()


class NestedClass2(Schema):
    DetalleIVA = fields.List(fields.Nested(BaseImponible))


class NestedListTest2(Schema):
    DesgloseIVA = fields.Nested(NestedClass2)


nested_list_model = NestedListTest2()
errors = nested_list_model.validate(new_obj)
if errors:
    raise Exception('Errors found 2', errors)
else:
    print 'Data 2'
    from pprintpp import pprint

    donald_dump = nested_list_model.dump(new_obj).data
    print donald_dump
    from lxml import etree
    from lxml import objectify

    xml_from_dict = dicttoxml(donald_dump, root=False, attr_type=False)
    print xml_from_dict
    xml_from_dict = xml_from_dict.replace('<item>', '')
    xml_from_dict = xml_from_dict.replace('</item>', '')
    print xml_from_dict
